/*-
 * Copyright 2017 UPLEX - Nils Goroll Systemoptimierung
 * All rights reserved.
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "vmod_pcre2.h"

#include "vcc_if.h"

VCL_INT
vmod_regex_info_int(VRT_CTX, struct vmod_pcre2_regex *regex, VCL_ENUM ints)
{
	uint32_t what, where;
	size_t sz;
	int ret;

	(void) ctx;
	CHECK_OBJ_NOTNULL(regex, VMOD_PCRE2_REGEX_MAGIC);

	if (strcmp(ints, "BACKREFMAX") == 0)
		what = PCRE2_INFO_BACKREFMAX;
	else if (strcmp(ints, "CAPTURECOUNT") == 0)
		what = PCRE2_INFO_CAPTURECOUNT;
	else if (strcmp(ints, "JITSIZE") == 0)
		what = PCRE2_INFO_JITSIZE;
	else if (strcmp(ints, "MATCHLIMIT") == 0)
		what = PCRE2_INFO_MATCHLIMIT;
	else if (strcmp(ints, "MAXLOOKBEHIND") == 0)
		what = PCRE2_INFO_MAXLOOKBEHIND;
	else if (strcmp(ints, "MINLENGTH") == 0)
		what = PCRE2_INFO_MINLENGTH;
	else if (strcmp(ints, "RECURSIONLIMIT") == 0)
		what = PCRE2_INFO_RECURSIONLIMIT;
	else if (strcmp(ints, "SIZE") == 0)
		what = PCRE2_INFO_SIZE;
	else
		WRONG("illegal enum in info_int");

	if (what == PCRE2_INFO_JITSIZE || what == PCRE2_INFO_SIZE) {
		ret = pcre2_pattern_info(regex->code, what, &sz);
		AZ(ret);
		return sz;
	}

	ret = pcre2_pattern_info(regex->code, what, &where);
	if (ret == PCRE2_ERROR_UNSET) {
		assert(what == PCRE2_INFO_MATCHLIMIT
		       || what == PCRE2_INFO_RECURSIONLIMIT);
		return -1;
	}
	AZ(ret);
	return where;
}
